//
//  main.c
//  Proiect PM
//
//  Created by Adrian Dudau on 15/05/2018.
//  Copyright © 2018 Adrian Dudau. All rights reserved.
//

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usart.h"

/* trebuie sa asteptam ca motoarele sa fie initializate cu 0 */
#define PWM_INIT_WAIT_MS 5700

/* viteza la care face avarii ledul */
#define HAZARD_LED_SPEED 30

#define DRIFT_SPEED 320

#define MOTOR_STOP 250
#define MOTOR_MIN 270
#define MOTOR_MAX 500
#define TURN_STEP 10
#define SPEED_STEP 20

/* Mesajele pe telecomanda bluetooth
 * L pentru apasare lunga
 * S pentru apasare scurta
 * La final se trimite END
 */
#define UP_L 'a'
#define DOWN_L 'c'
#define LEFT_L 'd'
#define RIGHT_L 'b'

#define UP_S 'A'
#define DOWN_S 'C'
#define LEFT_S 'D'
#define RIGHT_S 'B'

#define TRIANGLE_L 'e'
#define X_L 'g'
#define SQUARE_L 'h'
#define O_L 'f'

#define TRIANGLE_S 'E'
#define X_S 'G'
#define SQUARE_S 'H'
#define O_S 'F'

#define END '0'

/* PD4 - Motor rear left | OCR1B
 * PD5 - Motor rear right | OCR1A
 * PD7 - Board LED
 * PA0 - Arm LED
 * PA1 - Faruri
 */

/* variabile memorie motoare */
static volatile short Motor_mem = MOTOR_MIN;
static volatile char flash_time = 0;

void init_pins()
{
	/* Setez Ledul PA0 ca iesire. */
	DDRA |= (1 << PA0);

	/* Setez Ledul PA1 ca iesire. */
	DDRA |= (1 << PA1);

	/* Setez Ledul PD7 ca iesire. */
	DDRD |= (1 << PD7);

	/* BTN este intrare pe PD6 cu pull-up*/
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);

	/* Setez Motoarele ca iesire. */
	DDRD |= (1 << PD4);
	DDRD |= (1 << PD5);

	/* Fac intreruperi externe pt BTN/PD6 */
	sei();
	PCICR |= (1 << PCIE3);
	PCMSK3 |= (1 << PCINT30);
}

void init_timer0()
{
	/* intrerupere la ovf */
	TIMSK0 |= (1 << TOIE0);

	/* vreau prescalerul cel mai mare */
	TCCR0B |= (1 << CS02) | (1 << CS00);
}

void init_motors()
{
	PORTA |= (1 << PA0);

	/* Configurare TIMER1 PWM pt motoare brushless */
	/* NON Inverted PWM */
	TCCR1A |= (1<<COM1A1) | (1<<COM1B1) | (1<<WGM11);

	/* PRESCALER=64 MODE 14(FAST PWM) */
	TCCR1B |= (1<<WGM13) | (1<<WGM12) | (1<<CS11) | (1<<CS10);

	/* fPWM=50Hz (Perioada = 20ms Standard) */
	ICR1 = 4999;

	OCR1A = MOTOR_STOP;
	OCR1B = MOTOR_STOP;

	_delay_ms(PWM_INIT_WAIT_MS);

	PORTA &= ~(1 << PA0);
}

void headlights()
{
	PORTA ^= (1 << PA1);
}

void hazard_on()
{
	/* intrerupere la ovf */
	TIMSK0 |= (1 << TOIE0);
}

void hazard_off()
{
	TIMSK0 &= ~(1 << TOIE0);
	PORTA &= ~(1 << PA1);
}

void turn_left()
{
	PORTA |= (1 << PA0);

	OCR1A = Motor_mem + TURN_STEP;
	OCR1B = MOTOR_STOP;
}

void turn_right()
{
	PORTA |= (1 << PA0);

	OCR1A = MOTOR_STOP;
	OCR1B = Motor_mem + TURN_STEP;
}

void coast()
{
	PORTA &= ~(1 << PA0);

	OCR1A = MOTOR_STOP;
	OCR1B = MOTOR_STOP;
}

void forward()
{
	PORTA |= (1 << PA0);

	OCR1A = Motor_mem;
	OCR1B = Motor_mem;
}

void faster()
{
	if (Motor_mem + SPEED_STEP <= MOTOR_MAX) {
		Motor_mem += SPEED_STEP;

		OCR1A = Motor_mem;
		OCR1B = Motor_mem;
	}

	if (Motor_mem >= DRIFT_SPEED) {
		hazard_on();
	}
}

void slower()
{
	if (Motor_mem - SPEED_STEP >= MOTOR_MIN) {
		Motor_mem -= SPEED_STEP;

		OCR1A = Motor_mem;
		OCR1B = Motor_mem;
	}

	if (Motor_mem < DRIFT_SPEED) {
		hazard_off();
	}
}

void set_min()
{
	PORTA |= (1 << PA0);
	hazard_off();
	Motor_mem = MOTOR_MIN;
}

void set_max()
{
	PORTA |= (1 << PA0);
	hazard_on();
	Motor_mem = MOTOR_MAX;
}

/* Intrerupere pe receiverul bluetooth */
ISR(USART0_RX_vect) {
	/* Comanda cu un singur caracter */
	char c = USART0_receive();

	switch (c) {
		case UP_S:
			faster();
			break;

		case DOWN_S:
			slower();

		case RIGHT_S:
		case RIGHT_L:
			turn_right();
			break;

		case LEFT_S:
		case LEFT_L:
			turn_left();
			break;
			break;

		case X_S:
		case X_L:
			forward();
			break;

		case O_S:
			set_max();
			break;

		case TRIANGLE_S:
			headlights();
			break;

		case SQUARE_S:
			set_min();
			break;

		case END:
			coast();
			break;

		default:
			coast();
			break;
	}
}

ISR(TIMER0_OVF_vect)
{
	if (flash_time == HAZARD_LED_SPEED) {
		headlights();
		flash_time = 0;
	} else {
		flash_time++;
	}
}

int main(void)
{
	init_pins(); // neaparat primul

	init_timer0();

	USART0_init();

	init_motors();

	/* ready to use */
	PORTD |= (1 << PD7);
	hazard_off();

	while(1) {}

	cli();

	return 0;
}
